from aiohttp import ClientResponse, web_exceptions

__all__ = [
    "TokenRequired",
    "TokenValidationError",
]


def handle_error(response: ClientResponse, data: dict):
    base = {
        "headers": response.headers,
        "reason": str(data),
        "text": str(data.get("detail")),
    }
    if response.status == 401:
        raise web_exceptions.HTTPUnauthorized(**base)
    elif response.status == 429:
        raise web_exceptions.HTTPTooManyRequests(**base)
    elif response.status == 500:
        raise web_exceptions.HTTPInternalServerError(**base)
    else:
        base_error = web_exceptions.HTTPClientError
        base_error.status_code = response.status
        raise base_error(**base)


class TokenRequired(Exception):
    pass


class TokenValidationError(Exception):
    pass



from __future__ import annotations

from datetime import datetime
from io import BytesIO
from pathlib import Path
from typing import Any, Dict, List, Optional, Union
from uuid import UUID

import aiofiles
from aiohttp import ClientSession
from pydantic import AnyUrl, BaseModel, Field, HttpUrl

from .enums import *
from .errors import handle_error


class ModelCreator(BaseModel):
    username: str
    image: Optional[AnyUrl]


class ModelFileMetadata(BaseModel):
    format: Optional[str] = None
    fp: Optional[str] = None
    size: Optional[str] = None


class ModelImage(BaseModel):
    url: AnyUrl
    nsfw: Union[bool, str]
    width: int
    height: int
    generationProcess: Optional[str] = None


class ModelFile(BaseModel):
    name: str
    id: int
    sizeKb: float
    type: str
    pickleScanResult: Optional[str] = None
    pickleScanMessage: Optional[str] = None
    virusScanResult: Optional[str] = None
    scannedAt: Union[datetime, str, None] = ""
    downloadUrl: Optional[AnyUrl] = None
    format: Optional[str] = None
    metadata: Optional[ModelFileMetadata] = None


class ModelVersion(BaseModel):
    id: int
    modelId: int
    name: str
    baseModel: CivitAiBaseModelType  # noqa: 405
    description: Optional[str] = None
    trainedWords: List[str]
    createdAt: Optional[datetime] = None
    files: List[ModelFile]
    images: List[ModelImage]


class ModelStats(BaseModel):
    downloadCount: Optional[int] = None
    favoriteCount: Optional[int] = None
    commentCount: Optional[int] = None
    ratingCount: Optional[int] = None
    rating: Optional[float] = None


class MetadataItemVersion(BaseModel):
    name: str
    id: str
    createdAt: Optional[datetime] = None
    modelMetadataItemId: str
    externalId: Union[int, str]
    baseModel: CivitAiBaseModelType  # noqa: 405
    description: str
    downloadUrl: AnyUrl
    images: List[ModelImage]
    files: List[ModelFile]


class MetadataItem(BaseModel):
    id: str
    name: str
    activeVersionId: Optional[str] = None
    activeVersion: Union[MetadataItemVersion, None] = None
    author: Optional[str] = None
    authorAvatarUrl: Optional[AnyUrl] = None
    externalId: Union[int, str]
    type: CivitAiModelType  # noqa: 405
    allowCommercialUse: str
    allowNoCredit: bool
    nsfw: bool
    description: Optional[str] = None
    requestingUserId: str
    createdAt: Optional[datetime] = None
    ratings: ModelStats
    downloadStatus: DownloadStatus  # noqa
    tags: List[str]
    trainedWords: List[str]
    thumbnailImageUrl: Optional[AnyUrl] = None
    thumbnailImageNsfw: Optional[bool] = None
    versionMetadataItems: List[MetadataItemVersion]
    volumePath: Optional[str] = None
    modelCheckpointFilename: Optional[str] = None
    modelProvider: Optional[str] = None
    configYaml: Optional[str] = None
    datetimeDeleted: Optional[datetime] = None


class MetadataItems(BaseModel):
    items: List[MetadataItem]
    paginationMetadata: RangePaginationMetadata

    def first(self):
        return self.items[0]


class Model(BaseModel):
    id: int
    name: str
    description: str
    creator: ModelCreator
    type: CivitAiModelType  # noqa: 405
    nsfw: bool
    allowNoCredit: bool
    allowCommercialUse: str
    allowDerivatives: bool
    allowDifferentLicense: bool
    modelVersions: List[ModelVersion]
    stats: ModelStats
    tags: List[str]


class ModelsMetadata(BaseModel):
    totalItems: int
    currentPage: int
    pageSize: int
    totalPages: int
    nextPage: Optional[AnyUrl] = None


class Models(BaseModel):
    items: List[Model]
    metadata: ModelsMetadata

    def first(self):
        return self.items[0]


class CannyEdgesPayload(BaseModel):
    lowThreshold: int = Field(ge=1, le=255, title="Lowthreshold")
    upperThreshold: int = Field(ge=1, le=255, title="Upperthreshold")


class Esrgan4xUpscalingParams(BaseModel):
    parentInferenceId: Optional[str] = Field(None, title="Parentinferenceid")
    imageUrl: str = Field(title="Imageurl")
    scale: Optional[float] = Field(4.0, title="Scale")
    faceEnhance: Optional[bool] = Field(False, title="Faceenhance")


class FaceRestoreParams(BaseModel):
    parentInferenceId: Optional[str] = Field(None, title="Parentinferenceid")
    imageUrl: str = Field(title="Imageurl")
    fidelity: Optional[float] = Field(0.5, title="Fidelity")
    backgroundEnhance: Optional[bool] = Field(False, title="Backgroundenhance")
    faceUpsample: Optional[bool] = Field(False, title="Faceupsample")
    upscale: Optional[int] = Field(2, title="Upscale")


class HedPayload(BaseModel):
    scribble: bool = Field(title="Scribble")


class HighResFixParams(BaseModel):
    enabled: Optional[bool] = Field(False, title="Enabled")
    imageStrength: Optional[float] = Field(0.2, ge=0.0, le=0.9, title="Imagestrength")
    steps: Optional[int] = Field(25, ge=1, le=75, title="Steps")


class ImageRecord(BaseModel):
    id: str = Field(title="Id")
    folderPath: str = Field(title="Folderpath")
    filename: str = Field(title="Filename")
    createdAt: str = Field(title="Createdat")
    userId: str = Field(title="Userid")
    inferenceJobId: str = Field(title="Inferencejobid")
    favorite: bool = Field(title="Favorite")
    nsfw: bool = Field(title="Nsfw")

    @classmethod
    async def _download_file_bytes_io(
        cls,
        destination: BytesIO,
        seek: bool,
        bytes_: bytes,
    ) -> BytesIO:
        destination.write(bytes_)
        destination.flush()
        if seek is True:
            destination.seek(0)
        return destination

    @classmethod
    async def _download_file(
        cls,
        destination: Union[str, Path],
        bytes_: bytes,
    ) -> None:
        async with aiofiles.open(destination, "wb") as f:
            await f.write(bytes_)

    async def save(
        self,
        destination: Optional[Union[BytesIO, Path, str]] = None,
        seek: bool = True,
    ) -> Optional[BytesIO]:
        if destination is None:
            destination = BytesIO()

        async with ClientSession() as session:
            res = await session.get(self.get_url())
            if not res.status == 200:
                return handle_error(res, await res.json())

            bytes_ = await res.read()
            if isinstance(destination, (str, Path)):
                await self._download_file(destination, bytes_)
                return None
            return await self._download_file_bytes_io(destination, seek, bytes_)

    def get_url(self) -> HttpUrl:
        return f"https://ik.imagekit.io/hb42m9hh0/{self.folderPath}/{self.filename}"


class LoraParams(BaseModel):
    id: str = Field(title="Id")
    weight: float = Field(ge=0, le=1, title="Weight")


class MidasDepthPayload(BaseModel):
    surfaceNormalAngleRadians: float = Field(title="Surfacenormalangleradians")
    backgroundThreshold: float = Field(ge=0.0, le=1.0, title="Backgroundthreshold")
    depthAndNormal: bool = Field(title="Depthandnormal")


class MlsdPayload(BaseModel):
    valueThreshold: float = Field(ge=0.0, le=2.0, title="Valuethreshold")
    distanceThreshold: float = Field(ge=1, le=20, title="Distancethreshold")


class ModelDownloadRequestParams(BaseModel):
    externalId: str = Field(title="Externalid")
    modelVersionExternalId: Optional[str] = Field(None, title="Modelversionexternalid")


class OpenposePayload(BaseModel):
    includeFace: bool = Field(title="Includeface")
    includeBody: bool = Field(title="Includebody")
    includeHands: bool = Field(title="Includehands")


class PreprocessingResult(BaseModel):
    imageDataUri: str = Field(title="Imagedatauri")


class RangePaginationMetadata(BaseModel):
    currentPage: int = Field(title="Currentpage")
    pageSize: int = Field(title="Pagesize")
    totalItems: Optional[int] = Field(None, title="Totalitems")


class UpdateImageParams(BaseModel):
    favorite: Optional[bool] = Field(None, title="Favorite")
    nsfw: Optional[bool] = Field(None, title="Nsfw")


class WebhookEvent(BaseModel):
    id: str = Field(title="Id")
    type: str = Field(title="Type")
    data: Optional[Dict[str, Any]] = Field({}, title="Data")


class PreprocessingParams(BaseModel):
    preprocessingTechnique: PreprocessingTechnique  # noqa: F405
    baseImageUrl: Optional[str] = Field(None, title="Baseimageurl")
    preprocessingPayload: Optional[
        Union[
            CannyEdgesPayload,
            OpenposePayload,
            MlsdPayload,
            HedPayload,
            MidasDepthPayload,
        ]
    ] = Field(None, title="Preprocessingpayload")
    controlnetConditioningScale: Optional[float] = Field(
        1, ge=0.1, le=2.0, title="Controlnetconditioningscale"
    )
    controlGuidanceStart: Optional[float] = Field(
        [0.0], ge=0.0, le=1.0, title="Controlguidancestart"
    )
    controlGuidanceEnd: Optional[float] = Field(
        [1.0], ge=0.0, le=1.0, title="Controlguidanceend"
    )
    preprocessedImageUrl: Optional[str] = Field(None, title="Preprocessedimageurl")


class User(BaseModel):
    id: str = Field(title="Id")
    username: Optional[str] = Field(None, title="Username")
    fullName: Optional[str] = Field(None, title="Fullname")
    avatarUrl: Optional[str] = Field(None, title="Avatarurl")
    createdAt: datetime = Field(title="Createdat")
    nsfwEnabled: Optional[bool] = Field(False, title="Nsfwenabled")
    nsfwDefaultUnblur: Optional[bool] = Field(False, title="Nsfwdefaultunblur")
    nsfwShowModels: Optional[bool] = Field(False, title="Nsfwshowmodels")
    email: str = Field(title="Email")
    status: AccountStatus  # noqa: F405
    tosAckVersion: Optional[str] = Field(None, title="Tosackversion")
    lastAccess: Optional[datetime] = Field(None, title="Lastaccess")
    role: Optional[UserRole] = None  # noqa: F405
    contentDeletedAt: Optional[datetime] = Field(None, title="Contentdeletedat")


class CreateInferenceParams(BaseModel):
    parentInferenceId: Optional[str] = Field(None, title="Parentinferenceid")
    modelId: str = Field(title="Modelid")
    prompt: str = Field(title="Prompt")
    negativePrompt: Optional[str] = Field("", title="Negativeprompt")
    baseImageUrl: Optional[str] = Field(None, title="Baseimageurl")
    maskImageUrl: Optional[str] = Field(None, title="Maskimageurl")
    inpaintingMaskBlur: Optional[float] = Field(
        3.0, ge=0, le=10, title="Inpaintingmaskblur"
    )
    outputWpx: Optional[int] = Field(512, ge=1, le=1024, title="Outputwpx")
    outputHpx: Optional[int] = Field(512, ge=1, le=1024, title="Outputhpx")
    numImagesToGenerate: Optional[int] = Field(1, title="Numimagestogenerate")
    numInferenceSteps: Optional[int] = Field(25, ge=1, le=75, title="Numinferencesteps")
    samplingMethod: Optional[SamplingMethod] = "EULER"  # noqa: F405
    vae: Optional[VariationalAutoEncoder] = "stabilityai/sd-vae-ft-mse"  # noqa: F405
    lora: Optional[LoraParams] = None
    embeddingIds: Optional[List[str]] = Field([], title="Embeddingids")
    guidanceScale: Optional[float] = Field(7.0, ge=1, le=20, title="Guidancescale")
    strength: Optional[float] = Field(0.5, title="Strength", le=0.9, ge=0.0)
    clipSkip: Optional[int] = Field(1, ge=1, le=5, title="Clipskip")
    seed: Optional[int] = Field(None, title="Seed")
    controlNetPayloads: Optional[List[PreprocessingParams]] = Field(
        [], title="Controlnetpayloads"
    )
    highResFix: Optional[HighResFixParams] = None


class Inference(BaseModel):
    inferenceId: UUID = Field(title="Inferenceid")
    userId: UUID = Field(title="Userid")
    inferenceType: InferenceType  # noqa: F405
    inferencePayload: Union[
        CreateInferenceParams,
        FaceRestoreParams,
        Esrgan4xUpscalingParams,
    ] = Field(title="Inferencepayload")
    createdAt: datetime = Field(title="CreatedAt")
    dequeuedAt: Optional[datetime] = Field(None, title="DequeuedAt")
    completedAt: Optional[datetime] = Field(None, title="CompletedAt")
    priority: Optional[QueuePriority] = None  # noqa: F405
    status: InferenceStatus  # noqa: F405
    violatesTos: bool = Field(title="Violatestos")
    parentInferenceId: Optional[UUID] = Field(None, title="Parentinferenceid")


class InferenceHistoricalResult(BaseModel):
    inferenceId: UUID = Field(title="Inferenceid")
    userId: UUID = Field(title="Userid")
    inferencePayload: Union[
        CreateInferenceParams,
        FaceRestoreParams,
        Esrgan4xUpscalingParams,
    ] = Field(title="Inferencepayload")
    images: List[ImageRecord] = Field(title="Images")
    status: InferenceStatus  # noqa: F405
    inferenceType: InferenceType  # noqa: F405
    parentInferenceId: Optional[UUID] = Field(None, title="Parentinferenceid")


class UpdateInferenceParams(BaseModel):
    violatesTos: Optional[bool] = Field(None, title="Violatestos")
    status: Optional[InferenceStatus] = None  # noqa: F405
    inferencePayload: Optional[
        Union[CreateInferenceParams, FaceRestoreParams, Esrgan4xUpscalingParams]
    ] = Field(None, title="Inferencepayload")


class ApiPaginatedListResponseInferenceHistoricalResult(BaseModel):
    items: List[InferenceHistoricalResult] = Field(title="Items")
    paginationMetadata: RangePaginationMetadata



import asyncio
from typing import List, Union

from aiohttp import ClientResponse, ClientSession

from . import __version__
from ._token import validate_token
from .enums import (
    CivitAiBaseModelType,
    CivitAiModelType,
    CivitAiSortByType,
    DownloadStatus,
    InferenceStatus,
)
from .errors import TokenRequired, handle_error
from .models import (
    ApiPaginatedListResponseInferenceHistoricalResult,
    CreateInferenceParams,
    Inference,
    InferenceHistoricalResult,
    MetadataItem,
    MetadataItems,
    Models,
)

__all__ = [
    "ClientAPI",
]


def token_required(func):
    def wrapper(*args, **kwargs):
        token = args[0]._token
        if token is None:
            raise TokenRequired("This function need a token.")

        return func(*args, **kwargs)

    return wrapper


class ClientAPI:
    _api_version = "v1"
    _api_host = "easel-fgiw.onrender.com"

    def __init__(
        self,
        token: Union[str, None] = None,
        skip_token_validation: bool = False,
    ) -> None:
        if token is not None and not skip_token_validation:
            validate_token(token)

        self._token = token
        self._base_headers = {
            "Content-Type": "application/json",
            "User-Agent": "HappyAccidentsAPI/" + __version__,
            "Accept-Encoding": "gzip, deflate, br",
            "Connection": "keep-alive",
        }
        if self._token is not None:
            self._base_headers.update({"Authorization": "Bearer " + self._token})

    def _resolve_session(self, **kwargs):
        return ClientSession(headers=self._base_headers, **kwargs)

    def _get_url(self):
        return f"https://{self._api_host}/{self._api_version}"

    async def _proceed_response(self, response: ClientResponse):
        data = await response.json()
        if response.status >= 400:
            handle_error(response, data)

        return data

    async def fetch_inference(
        self,
        inference_id: int,
    ) -> InferenceHistoricalResult:
        async with self._resolve_session() as session:
            response = await session.get(f"{self._get_url()}/inferences/{inference_id}")
            data = await self._proceed_response(response)
            return InferenceHistoricalResult.model_validate(data)

    @token_required
    async def fetch_inferences(
        self,
        current_page: int = 0,
        page_size: int = 10,
    ) -> ApiPaginatedListResponseInferenceHistoricalResult:
        async with self._resolve_session() as session:
            response = await session.get(
                f"{self._get_url()}/inferences/",
                params={
                    "current_page": current_page,
                    "page_size": page_size,
                },
            )
            data = await self._proceed_response(response)
            return ApiPaginatedListResponseInferenceHistoricalResult.model_validate(
                data
            )

    @token_required
    async def fetch_community_inferences(
        self,
        current_page: int = 0,
        page_size: int = 10,
        favorites_only: bool = True,
        nsfw: bool = False,
        query: str = "",
    ) -> ApiPaginatedListResponseInferenceHistoricalResult:
        async with self._resolve_session() as session:
            response = await session.get(
                f"{self._get_url()}/community/inferences/",
                params={
                    "current_page": current_page,
                    "page_size": page_size,
                    "favorites_only": str(favorites_only),
                    "nsfw": str(nsfw),
                    "search": query,
                },
            )
            data = await self._proceed_response(response)
            return ApiPaginatedListResponseInferenceHistoricalResult.model_validate(
                data
            )

    @token_required
    async def create_inference(
        self,
        inference_params: CreateInferenceParams,
        wait_for_response: bool = True,
    ) -> Union[InferenceHistoricalResult, Inference]:
        async with self._resolve_session() as session:
            response = await session.post(
                f"{self._get_url()}/inference/",
                json=inference_params.model_dump(),
            )
            data = await self._proceed_response(response)
            inference = Inference.model_validate(data)

            if not wait_for_response:
                return inference

            while True:
                await asyncio.sleep(5)

                inference_result = await self.fetch_inference(inference.inferenceId)
                if inference_result is None:
                    continue

                if inference_result.status in (
                    InferenceStatus.FAILED,
                    InferenceStatus.COMPLETED,
                ):
                    return inference_result

    @token_required
    async def fetch_max_queue_depth(self) -> int:
        async with self._resolve_session() as session:
            response = await session.get(f"{self._get_url()}/max_queue_depth")
            data = await self._proceed_response(response)
            return data

    async def fetch_models(
        self,
        query: str = "",
        model_types: List[CivitAiModelType] = ["Checkpoint"],
        sort_by: CivitAiSortByType = "Highest Rated",
        nsfw: bool = False,
        current_page: int = 1,
        page_size: int = 10,
    ) -> Models:
        async with self._resolve_session() as session:
            response = await session.get(
                f"{self._get_url()}/models/models",
                params={
                    "searchQuery": query,
                    "modelType": model_types,
                    "sortBy": sort_by,
                    "nsfw": str(nsfw),
                    "current_page": current_page,
                    "page_size": page_size,
                },
            )
            data = await self._proceed_response(response)
            return Models.model_validate(data)

    async def fetch_metadata_items(
        self,
        query: str = "",
        sort_by: CivitAiSortByType = "Highest Rated",
        nsfw: bool = False,
        current_page: int = 1,
        page_size: int = 10,
        download_status: DownloadStatus = "COMPLETED",
        model_types: List[CivitAiModelType] = ["Checkpoint"],
        base_models: List[CivitAiBaseModelType] = [],
        user: Union[str, None] = None,
    ) -> MetadataItems:
        async with self._resolve_session() as session:
            params = {
                "search": query,
                "sortBy": sort_by,
                "nsfw": str(nsfw),
                "currentPage": current_page,
                "pageSize": page_size,
                "downloadStatus": download_status,
                "modelType": model_types,
                "baseModel": base_models,
            }
            if user is not None:
                params["user"] = user

            response = await session.get(
                f"{self._get_url()}/models/metadata-items",
                params=params,
            )
            data = await self._proceed_response(response)
            return MetadataItems.model_validate(data)

    async def fetch_metadata_item(
        self,
        model_metadata_id: str,
    ) -> MetadataItem:
        async with self._resolve_session() as session:
            response = await session.get(
                f"{self._get_url()}/models/metadata-items/{model_metadata_id}",
            )
            data = await self._proceed_response(response)
            return MetadataItem.model_validate(data)

    async def fetch_inference_result_by_image_id(
        self,
        image_uuid: str,
    ) -> InferenceHistoricalResult:
        async with self._resolve_session() as session:
            response = await session.get(
                f"{self._get_url()}/images/{image_uuid}",
            )
            data = await self._proceed_response(response)
            return InferenceHistoricalResult.model_validate(data)



"""
HappyAccidents API Wrapper

:license: Apache License 2.0, see LICENSE for more details.
"""

__title__ = "happyaccidentsapi"
__author__ = "hoopengo"
__license__ = "Apache License 2.0"
__version__ = "0.0.0"

from ._happyaccidentsapi import *
from .enums import *
from .models import *



from enum import Enum


class AccountStatus(str, Enum):
    ACTIVE = "ACTIVE"
    BANNED = "BANNED"


class CivitAiBaseModelType(str, Enum):
    SD14 = "SD 1.4"
    SD15 = "SD 1.5"
    SD20 = "SD 2.0"
    SD20_768 = "SD 2.0 768"
    SD21 = "SD 2.1"
    SD21_768 = "SD 2.1 768"
    OTHER = "Other"


class CivitAiModelType(str, Enum):
    CHECKPOINT = "Checkpoint"
    TEXTUAL_INVERSION = "TextualInversion"
    HYPERNETWORK = "Hypernetwork"
    AESTHETIC_GRADIENT = "AestheticGradient"
    LORA = "LORA"
    LOCON = "LoCon"
    CONTROLNET = "Controlnet"
    POSES = "Poses"
    OTHER = "Other"
    WILCARDS = "Wildcards"


class CivitAiSortByType(str, Enum):
    HIGHEST_RATED = "Highest Rated"
    MOST_DOWNLOADED = "Most Downloaded"
    NEWEST = "Newest"
    FAVORITED = "Favorited"


class DownloadStatus(str, Enum):
    QUEUE = "QUEUED"
    DOWNLOADING = "DOWNLOADING"
    CONVERTING = "CONVERTING"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"


class InferenceStatus(str, Enum):
    PENDING = "PENDING"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"


class InferenceType(str, Enum):
    FACE_RESTORE = "FACE_RESTORE"
    UPSCALING = "UPSCALING"
    IMAGE_TO_IMAGE = "IMAGE_TO_IMAGE"
    TEXT_TO_IMAGE = "TEXT_TO_IMAGE"
    INPAINTING = "INPAINTING"
    CONTROLNET_TXT2IMG = "CONTROLNET_TXT2IMG"
    CONTROLNET_IMG2IMG = "CONTROLNET_IMG2IMG"
    CONTROLNET_INPAINTING = "CONTROLNET_INPAINTING"


class NotificationStatus(str, Enum):
    PENDING = "PENDING"
    APPROVED = "APPROVED"
    REJECTED = "REJECTED"
    SEND_SUCCESSFUL = "SEND_SUCCESSFUL"
    SEND_FAILURE = "SEND_FAILURE"


class PreprocessingTechnique(str, Enum):
    CANNY_EDGES = "CANNY_EDGES"
    OPENPOSE = "OPENPOSE"
    SCRIBBLE = "SCRIBBLE"
    SEGMENT = "SEGMENT"
    MLSD = "MLSD"
    HED = "HED"
    MIDAS_DEPTH = "MIDAS_DEPTH"
    REFERENCE_ONLY = "REFERENCE_ONLY"
    TILE = "TILE"


class QueuePriority(str, Enum):
    TEN_MINUTES = "10m"
    ONE_MINUTE = "1m"
    TEN_SECONDS = "10s"
    ONE_SECOND = "1s"


class SamplingMethod(str, Enum):
    EULER = "EULER"
    EULER_A = "EULER_A"
    LMS = "LMS"
    LMS_KARRAS = "LMS_KARRAS"
    DDPM = "DDPM"
    DDIM = "DDIM"
    DPM_SOLVER_MULTISTEP = "DPM_SOLVER_MULTISTEP"
    DPM_SOLVER_MULTISTEP_KARRAS = "DPM_SOLVER_MULTISTEP_KARRAS"
    DPM_SOLVER_MULTISTEP_2PLUS = "DPM_SOLVER_MULTISTEP_++"
    DPM_SOLVER_MULTISTEP_2PLUS_KARRAS = "DPM_SOLVER_MULTISTEP_++_KARRAS"
    DPM_2PLUS_SDE = "DPM_++_SDE"
    DPM_2PLUS_SDE_KARRAS = "DPM_++_SDE_KARRAS"
    UNI_PC = "UNI_PC"
    DPM_DISCRETE_SCHEDULER = "DPM_DISCRETE_SCHEDULER"
    DPM_DISCRETE_SCHEDULER_ANCESTRAL = "DPM_DISCRETE_SCHEDULER_ANCESTRAL"


class VariationalAutoEncoder(str, Enum):
    SD_VAE_FT_EMA = "stabilityai/sd-vae-ft-ema"
    SD_VAE_FT_MSE = "stabilityai/sd-vae-ft-mse"
    ABYSS_ORANGE_MIX = "abyss-orange-mix"
    KL_F8_ANIME2 = "kl-f8-anime2"


class UserRole(str, Enum):
    ADMIN = "ADMIN"
    MOD = "MOD"



from functools import lru_cache

from .errors import TokenValidationError

__all__ = []


@lru_cache()
def validate_token(token: str) -> bool:
    """
    Validate Telegram token
    """
    if not isinstance(token, str):
        raise TokenValidationError(
            f"Token is invalid! It must be 'str' type instead of {type(token)} type."
        )

    if any(x.isspace() for x in token):
        message = "Token is invalid! It can't contains spaces."
        raise TokenValidationError(message)

    return True
